@@ -1,31 +1,20 @@
 """quixote.http_response
-$HeadURL: svn+ssh://svn/repos/trunk/quixote/http_response.py $
-$Id: http_response.py 25234 2004-09-30 17:36:19Z nascheme $
+$HeadURL: svn+ssh://svn.mems-exchange.org/repos/trunk/quixote/http_response.py $
+$Id: http_response.py 27524 2005-10-06 19:56:57Z dbinger $
 
 Provides the HTTPResponse class.
-
-Derived from Zope's HTTPResponse module (hence the different
-copyright and license from the rest of Quixote).
 """
 
-##############################################################################
-#
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-#
-# This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
-#
-##############################################################################
-
-__revision__ = "$Id: http_response.py 25234 2004-09-30 17:36:19Z nascheme $"
-
 import time
+from sets import Set
+try:
+    import zlib
+except ImportError:
+    pass
+import struct
 from rfc822 import formatdate
-from types import StringType, IntType
+import quixote
+from quixote.html import stringify
 
 status_reasons = {
     100: 'Continue',
@@ -76,6 +65,23 @@ status_reasons = {
     507: 'Insufficient Storage',
 }
 
+_GZIP_HEADER = ("\037\213" # magic
+                "\010" # compression method
+                "\000" # flags
+                "\000\000\000\000" # time, who cares?
+                "\002"
+                "\377")
+
+_GZIP_EXCLUDE = Set(["application/pdf",
+                     "application/zip",
+                     "audio/mpeg",
+                     "image/gif",
+                     "image/jpeg",
+                     "image/png",
+                     "video/mpeg",
+                     "video/quicktime",
+                     "video/x-msvideo",
+                     ])
 
 class HTTPResponse:
     """
@@ -87,6 +93,12 @@ class HTTPResponse:
     after all).
 
     Instance attributes:
+      content_type : string
+        the MIME content type of the response (does not include extra params
+        like charset)
+      charset : string | None
+        the character encoding of the the response.  If none, the 'charset'
+        parameter of the Context-Type header will not be included.
       status_code : int
         HTTP response status code (integer between 100 and 599)
       reason_phrase : string
@@ -97,12 +109,9 @@ class HTTPResponse:
         by 'set_header()' goes here.  Does not include "Status" or
         "Set-Cookie" headers (unless someone uses set_header() to set
         them, but that would be foolish).
-      body : string
-        the response body, None by default.  If the body is never
-        set (ie. left as None), the response will not include
-        "Content-type" or "Content-length" headers.  These headers
-        are set as soon as the body is set (with set_body()), even
-        if the body is an empty string.
+      body : str | Stream
+        the response body, None by default.  Note that if the body is not a
+        stream then it is already encoded using 'charset'.
       buffered : bool
         if false, response data will be flushed as soon as it is
         written (the default is true).  This is most useful for
@@ -126,10 +135,18 @@ class HTTPResponse:
         else's problem.
     """
 
-    def __init__(self, status=200, body=None):
+    DEFAULT_CONTENT_TYPE = 'text/html'
+    DEFAULT_CHARSET = None # defaults to quixote.DEFAULT_CHARSET
+
+    
+    def __init__(self, status=200, body=None, content_type=None, charset=None):
         """
         Creates a new HTTP response.
         """
+        self.content_type = content_type or self.DEFAULT_CONTENT_TYPE
+        self.charset = (charset or
+                        self.DEFAULT_CHARSET or
+                        quixote.DEFAULT_CHARSET)
         self.set_status(status)
         self.headers = {}
 
@@ -140,9 +157,29 @@ class HTTPResponse:
 
         self.cookies = {}
         self.cache = 0
-        self.buffered = 1
+        self.buffered = True
         self.javascript_code = None
 
+    def set_content_type(self, content_type, charset=None):
+        """(content_type : string, charset : string = None)
+
+        Set the content type of the response to the MIME type specified by
+        'content_type'.  If 'charset' is not provided and the content_type is
+        text/* then the charset attribute remains unchanged, otherwise the
+        charset attribute is set to None and the charset parameter will not
+        be included as part of the Content-Type header.
+        """
+        content_type = content_type.lower()
+        if charset is not None or not content_type.startswith('text/'):
+            self.charset = charset
+        self.content_type = content_type
+
+    def set_charset(self, charset):
+        if not charset:
+            self.charset = None
+        else:
+            self.charset = str(charset).lower()
+
     def set_status(self, status, reason=None):
         """set_status(status : int, reason : string = None)
 
@@ -153,7 +190,7 @@ class HTTPResponse:
         reason phrase for its group of status codes will be used; eg.
         if status == 493, the reason for status 400 will be used.
         """
-        if type(status) is not IntType:
+        if not isinstance(status, int):
             raise TypeError, "status must be an integer"
         if not (100 <= status <= 599):
             raise ValueError, "status must be between 100 and 599"
@@ -187,31 +224,58 @@ class HTTPResponse:
         """
         return self.headers.get(name.lower(), default)
 
-    def set_content_type(self, ctype):
-        """set_content_type(ctype : string)
-
-        Set the "Content-type" header to the MIME type specified in ctype.
-        Shortcut for set_header("Content-type", ctype).
-        """
-        self.headers["content-type"] = ctype
-
-    def set_body(self, body):
-        """set_body(body : any)
-
-        Sets the return body equal to the argument "body". Also updates the
-        "Content-length" header if the length is of the body is known.  If
-        the "Content-type" header has not yet been set, it is set to
-        "text/html".
-        """
-        if isinstance(body, Stream):
-            self.body = body
-            if body.length is not None:
-                self.set_header('content-length', body.length)
+    def set_expires(self, seconds=0, minutes=0, hours=0, days=0):
+        if seconds is None:
+            self.cache = None # don't generate 'Expires' header
         else:
-            self.body = str(body)
-            self.set_header('content-length', len(self.body))
-        if not self.headers.has_key('content-type'):
-            self.set_header('content-type', 'text/html; charset=iso-8859-1')
+            self.cache = seconds + 60*(minutes + 60*(hours + 24*days))
+
+    def _encode_chunk(self, chunk):
+        """(chunk : str | unicode) -> str
+        """
+        if isinstance(chunk, unicode):
+            if self.charset is None:
+                # iso-8859-1 is the default for the HTTP protocol if charset
+                # parameter of content-type header is not provided
+                chunk = chunk.encode('iso-8859-1')
+            else:
+                chunk = chunk.encode(self.charset)
+        else:
+            # we assume that the str is in the correct encoding or does
+            # not contain character data
+            pass
+        return chunk
+
+    def _compress_body(self, body):
+        """(body: str) -> str
+        """
+        n = len(body)
+        co = zlib.compressobj(6, zlib.DEFLATED, -zlib.MAX_WBITS,
+                              zlib.DEF_MEM_LEVEL, 0)
+        chunks = [_GZIP_HEADER,
+                  co.compress(body),
+                  co.flush(),
+                  struct.pack("<ll", zlib.crc32(body), n)]
+        compressed_body = "".join(chunks)
+        ratio = float(n) / len(compressed_body)
+        #print "gzip original size %d, ratio %.1f" % (n, ratio)
+        if ratio > 1.0:
+            self.set_header("Content-Encoding", "gzip")
+            return compressed_body
+        else:
+            return body
+
+    def set_body(self, body, compress=False):
+        """(body : any, compress : bool = False)
+
+        Sets the response body equal to the argument 'body'.  If 'compress'
+        is true then the body may be compressed using 'gzip'.
+        """
+        if not isinstance(body, Stream):
+            body = self._encode_chunk(stringify(body))
+            if compress and self.content_type not in _GZIP_EXCLUDE:
+                body = self._compress_body(body)
+        self.body = body
 
     def expire_cookie(self, name, **attrs):
         """
@@ -269,9 +333,9 @@ class HTTPResponse:
         elif not self.javascript_code.has_key(code_id):
             self.javascript_code[code_id] = code
 
-    def redirect(self, location, permanent=0):
+    def redirect(self, location, permanent=False):
         """Cause a redirection without raising an error"""
-        if not isinstance(location, StringType):
+        if not isinstance(location, str):
             raise TypeError, "location must be a string (got %s)" % `location`
         # Ensure that location is a full URL
         if location.find('://') == -1:
@@ -285,38 +349,46 @@ class HTTPResponse:
         self.set_content_type('text/plain')
         return "Your browser should have redirected you to %s" % location
 
+    def get_status_code(self):
+        return self.status_code
+
+    def get_reason_phrase(self):
+        return self.reason_phrase
+
+    def get_content_type(self):
+        return self.content_type
+
+    def get_content_length(self):
+        if self.body is None:
+            return None
+        elif isinstance(self.body, Stream):
+            return self.body.length
+        else:
+            return len(self.body)
+
     def _gen_cookie_headers(self):
         """_gen_cookie_headers() -> [string]
 
         Build a list of "Set-Cookie" headers based on all cookies
         set with 'set_cookie()', and return that list.
         """
-        cookie_list = []
-        for (name, attrs) in self.cookies.items():
-
-            # Note that as of May 98, IE4 ignores cookies with
-            # quoted cookie attr values, so only the value part
-            # of name=value pairs may be quoted.
-
-            # 'chunks' is a list of "name=val" chunks; will be joined
-            # with "; " to create the "Set-cookie" header.
-            chunks = ['%s="%s"' % (name, attrs['value'])]
-
-            for (name, val) in attrs.items():
+        cookie_headers = []
+        for name, attrs in self.cookies.items():
+            value = str(attrs['value'])
+            if '"' in value:
+                value = value.replace('"', '\\"')
+            chunks = ['%s="%s"' % (name, value)]
+            for name, val in attrs.items():
                 name = name.lower()
                 if val is None:
                     continue
                 if name in ('expires', 'domain', 'path', 'max_age', 'comment'):
                     name = name.replace('_', '-')
-                    chunks.append("%s=%s" % (name, val))
+                    chunks.append('%s=%s' % (name, val))
                 elif name == 'secure' and val:
                     chunks.append("secure")
-
-            cookie_list.append(("Set-Cookie", ("; ".join(chunks))))
-
-        # Should really check size of cookies here!
-
-        return cookie_list
+            cookie_headers.append(("Set-Cookie", '; '.join(chunks)))
+        return cookie_headers
 
     def generate_headers(self):
         """generate_headers() -> [(name:string, value:string)]
@@ -325,10 +397,6 @@ class HTTPResponse:
         """
         headers = []
 
-        # "Status" header must come first.
-        headers.append(("Status", "%03d %s" % (self.status_code,
-                                               self.reason_phrase)))
-
         for name, value in self.headers.items():
             headers.append((name.title(), value))
 
@@ -338,13 +406,13 @@ class HTTPResponse:
 
         # Date header
         now = time.time()
-        if not self.headers.has_key("date"):
+        if "date" not in self.headers:
             headers.append(("Date", formatdate(now)))
 
         # Cache directives
         if self.cache is None:
             pass # don't mess with the expires header
-        elif not self.headers.has_key("expires"):
+        elif "expires" not in self.headers:
             if self.cache > 0:
                 expire_date = formatdate(now + self.cache)
             else:
@@ -352,50 +420,63 @@ class HTTPResponse:
                                    # with some clients
             headers.append(("Expires", expire_date))
 
+        # Content-type
+        if "content-type" not in self.headers:
+            if self.charset is not None:
+                value = '%s; charset=%s' % (self.content_type, self.charset)
+            else:
+                value = '%s' % self.content_type
+            headers.append(('Content-Type', value))
+
+        # Content-Length
+        if "content-length" not in self.headers:
+            length = self.get_content_length()
+            if length is not None:
+                headers.append(('Content-Length', str(length)))
+
         return headers
 
-
-    def write(self, file):
-        """write(file : file)
-
-        Write the HTTP response headers and body to 'file'.  This is not
-        a complete HTTP response, as it doesn't start with a response
-        status line as specified by RFC 2616.  It does, however, start
-        with a "Status" header as described by the CGI spec.  It
-        is expected that this response is parsed by the web server
-        and turned into a complete HTTP response.
+    def generate_body_chunks(self):
+        """Return a sequence of body chunks, encoded using 'charset'.
         """
-        # XXX currently we write a response like this:
-        #  Status: 200 OK
-        #  Content-type: text/html; charset=iso-8859-1
-        #  Content-length: 100
-        #  Set-Cookie: foo=bar
-        #  Set-Cookie: bar=baz
-        #
-        #  <html><body>This is a document</body></html>
-        #
-        # which has to be interpreted by the web server to create
-        # a true HTTP response -- that is, this is for a
-        # "parsed header" CGI driver script.
-        #
-        # We should probably have provisions for operating in
-        # "non-parsed header" mode, where the CGI script is responsible
-        # for generating a complete HTTP response with no help from the
-        # server.
-        flush_output = not self.buffered and hasattr(file, 'flush')
+        if self.body is None:
+            pass
+        elif isinstance(self.body, Stream):
+            for chunk in self.body:
+                yield self._encode_chunk(chunk)
+        else:
+            yield self.body # already encoded
+
+    def write(self, output, include_status=True, include_body=True):
+        """(output:file, include_status:bool=True, include_body:bool=True)
+
+        Write the HTTP response headers and, by default, body to 'output'.
+        This is not a complete HTTP response, as it doesn't start with a
+        response status line as specified by RFC 2616.  By default, it
+        does start with a "Status" header as described by the CGI spec.
+        It is expected that this response is parsed by the web server and
+        turned into a complete HTTP response. If include_body is False,
+        only the headers are written to 'output'. This is used to support
+        HTTP HEAD requests.
+        """
+        flush_output = not self.buffered and hasattr(output, 'flush')
+        if include_status:
+            # "Status" header must come first.
+            output.write("Status: %03d %s\r\n" % (self.status_code,
+                                                  self.reason_phrase))
         for name, value in self.generate_headers():
-            file.write("%s: %s\r\n" % (name, value))
-        file.write("\r\n")
-        if self.body is not None:
-            if isinstance(self.body, Stream):
-                for chunk in self.body:
-                    file.write(chunk)
-                    if flush_output:
-                        file.flush()
-            else:
-                file.write(self.body)
+            output.write("%s: %s\r\n" % (name, value))
+        output.write("\r\n")
         if flush_output:
-            file.flush()
+            output.flush()
+        if not include_body:
+            return
+        for chunk in self.generate_body_chunks():
+            output.write(chunk)
+            if flush_output:
+                output.flush()
+        if flush_output:
+            output.flush()
 
 
 class Stream:
