@@ -1,11 +1,10 @@
 #!/usr/bin/env python
 """$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/quixote/server/cgi_server.py $
-$Id: cgi_server.py 25476 2004-10-27 21:54:59Z nascheme $
+$Id: cgi_server.py 27684 2005-11-10 15:25:17Z dbinger $
 """
 
 import sys
 import os
-from quixote.http_request import HTTPRequest
 
 def run(create_publisher):
     if sys.platform == "win32":
@@ -14,8 +13,7 @@ def run(create_publisher):
         msvcrt.setmode(sys.__stdin__.fileno(), os.O_BINARY)
         msvcrt.setmode(sys.__stdout__.fileno(), os.O_BINARY)
     publisher = create_publisher()
-    request = HTTPRequest(sys.__stdin__, os.environ)
-    response = publisher.process_request(request)
+    response = publisher.process(sys.__stdin__, os.environ)
     try:
         response.write(sys.__stdout__)
     except IOError, err:
