@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/quixote/server/fastcgi_server.py $
-$Id: fastcgi_server.py 25476 2004-10-27 21:54:59Z nascheme $
+$Id: fastcgi_server.py 27684 2005-11-10 15:25:17Z dbinger $
 
 Server for Quixote applications that use FastCGI.  It should work
 for CGI too but the cgi_server module is preferred as it is more
@@ -8,14 +8,12 @@ portable.
 """
 
 from quixote.server import _fcgi
-from quixote.http_request import HTTPRequest
 
 def run(create_publisher):
     publisher = create_publisher()
     while _fcgi.isFCGI():
         f = _fcgi.FCGI()
-        request = HTTPRequest(f.inp, f.env)
-        response = publisher.process_request(request)
+        response = publisher.process(f.inp, f.env)
         try:
             response.write(f.out)
         except IOError, err:
