@@ -1,6 +1,5 @@
-"""quixote.session
-$HeadURL: svn+ssh://svn/repos/trunk/quixote/session.py $
-$Id: session.py 25234 2004-09-30 17:36:19Z nascheme $
+"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/quixote/session.py $
+$Id: session.py 26524 2005-04-08 10:22:34Z dbinger $
 
 Quixote session management.  There are two levels to Quixote's
 session management system:
@@ -23,10 +22,32 @@ doc/session-mgmt.txt for information on session persistence.
 
 from time import time, localtime, strftime
 
-from quixote import get_publisher
-from quixote.errors import SessionError
+from quixote import get_publisher, get_cookie, get_response, get_request, \
+     get_session
 from quixote.util import randbytes
 
+class NullSessionManager:
+    """A session manager that does nothing.  It is the default session manager.
+    """
+
+    def start_request(self):
+        """
+        Called near the beginning of each request: after the HTTPRequest
+        object has been built, but before we traverse the URL or call the
+        callable object found by URL traversal.
+        """
+
+    def finish_successful_request(self):
+        """Called near the end of each successful request.  Not called if
+        there were any errors processing the request.
+        """
+
+    def finish_failed_request(self):
+        """Called near the end of a failed request (i.e. a exception that was
+        not a PublisherError was raised.
+        """
+
+
 class SessionManager:
     """
     SessionManager acts as a dictionary of all sessions, mapping session
@@ -54,14 +75,10 @@ class SessionManager:
 
 
     def __init__(self, session_class=None, session_mapping=None):
-        """SessionManager(session_class : class = Session,
-                          session_mapping : mapping = {})
+        """(session_class : class = Session, session_mapping : mapping = None)
 
         Create a new session manager.  There should be one session
-        manager per publisher (really SessionPublisher), ie. one
-        per process.  Note that SessionPublisher's constructor will
-        take care of creating a session manager for you if you don't
-        do it yourself.
+        manager per publisher, ie. one per process
 
         session_class is used by the new_session() method -- it returns
         an instance of session_class.
@@ -86,14 +103,14 @@ class SessionManager:
     # doesn't provide all of the mapping methods needed here)
 
     def keys(self):
-        """keys() -> [string]
+        """() -> [string]
 
         Return the list of session IDs of sessions in this session manager.
         """
         return self.sessions.keys()
 
     def sorted_keys(self):
-        """sorted_keys() -> [string]
+        """() -> [string]
 
         Return the same list as keys(), but sorted.
         """
@@ -102,14 +119,14 @@ class SessionManager:
         return keys
 
     def values(self):
-        """values() -> [Session]
+        """() -> [Session]
 
         Return the list of sessions in this session manager.
         """
         return self.sessions.values()
 
     def items(self):
-        """items() -> [(string, Session)]
+        """() -> [(string, Session)]
 
         Return the list of (session_id, session) pairs in this session
         manager.
@@ -117,7 +134,7 @@ class SessionManager:
         return self.sessions.items()
 
     def get(self, session_id, default=None):
-        """get(session_id : string, default : any = None) -> Session
+        """(session_id : string, default : any = None) -> Session
 
         Return the session object identified by 'session_id', or None if
         no such session.
@@ -125,7 +142,7 @@ class SessionManager:
         return self.sessions.get(session_id, default)
 
     def __getitem__(self, session_id):
-        """__getitem__(session_id : string) -> Session
+        """(session_id : string) -> Session
 
         Return the session object identified by 'session_id'.  Raise KeyError
         if no such session.
@@ -133,7 +150,7 @@ class SessionManager:
         return self.sessions[session_id]
 
     def has_key(self, session_id):
-        """has_key(session_id : string) -> boolean
+        """(session_id : string) -> boolean
 
         Return true if a session identified by 'session_id' exists in
         the session manager.
@@ -145,7 +162,7 @@ class SessionManager:
     has_session = has_key
 
     def __setitem__(self, session_id, session):
-        """__setitem__(session_id : string, session : Session)
+        """(session_id : string, session : Session)
 
         Store 'session' in the session manager under 'session_id'.
         """
@@ -157,7 +174,7 @@ class SessionManager:
         self.sessions[session_id] = session
 
     def __delitem__(self, session_id):
-        """__getitem__(session_id : string) -> Session
+        """(session_id : string) -> Session
 
         Remove the session object identified by 'session_id' from the session
         manager.  Raise KeyError if no such session.
@@ -172,21 +189,21 @@ class SessionManager:
     # request is finished processing.
 
     def abort_changes(self, session):
-        """abort_changes(session : Session)
+        """(session : Session)
 
         Placeholder for subclasses that implement transactional
         persistence: forget about saving changes to the current
-        session.  Called by SessionPublisher when a request fails,
+        session.  Called by the publisher when a request fails,
         ie. when it catches an exception other than PublishError.
         """
         pass
 
     def commit_changes(self, session):
-        """commit_changes(session : Session)
+        """(session : Session)
 
         Placeholder for subclasses that implement transactional
         persistence: commit changes to the current session.  Called by
-        SessionPublisher when a request completes successfully, or is
+        the publisher when a request completes successfully, or is
         interrupted by a PublishError exception.
         """
         pass
@@ -197,23 +214,22 @@ class SessionManager:
     # above mapping methods, and are concerned with all the high-
     # level details of managing web sessions
 
-    def new_session(self, request, id):
-        """new_session(request : HTTPRequest, id : string)
-           -> Session
+    def new_session(self, id):
+        """(id : string) -> Session
 
         Return a new session object, ie. an instance of the session_class
         class passed to the constructor (defaults to Session).
         """
-        return self.session_class(request, id)
+        return self.session_class(id)
 
-    def _get_session_id(self, request, config):
-        """_get_session_id(request : HTTPRequest) -> string
+    def _get_session_id(self, config):
+        """() -> string
 
         Find the ID of the current session by looking for the session
-        cookie in 'request'.  Return None if no such cookie or the
+        cookie in the request.  Return None if no such cookie or the
         cookie has been expired, otherwise return the cookie's value.
         """
-        id = request.cookies.get(config.session_cookie_name)
+        id = get_cookie(config.session_cookie_name)
         if id == "" or id == "*del*":
             return None
         else:
@@ -228,23 +244,18 @@ class SessionManager:
             id = randbytes(8)  # 64-bit random number
         return id
 
-    def _create_session(self, request):
+    def _create_session(self):
         # Create a new session object, with no ID for now - one will
         # be assigned later if we save the session.
-        return self.new_session(request, None)
+        return self.new_session(None)
 
-    def get_session(self, request):
-        """get_session(request : HTTPRequest) -> Session
+    def get_session(self):
+        """() -> Session
 
         Fetch or create a session object for the current session, and
         return it.  If a session cookie is found in the HTTP request
-        object 'request', use it to look up and return an existing
-        session object.  If no session cookie is found, create a new
-        session.  If the session cookie refers to a non-existent
-        session, raise SessionError.  If the check_session_addr config
-        variable is true, then a mismatch between the IP address stored
-        in an existing session the IP address of the current request
-        also causes SessionError.
+        object, use it to look up and return an existing session object.
+        If no session cookie is found, create a new session.
 
         Note that this method does *not* cause the new session to be
         stored in the session manager, nor does it drop a session cookie
@@ -252,43 +263,19 @@ class SessionManager:
         maintain_session(), called at the end of a request.
         """
         config = get_publisher().config
-        id = self._get_session_id(request, config)
-        if id is not None:
-            session = self.get(id)
-            if session is None:
-                # Note that it's important to revoke the session cookie
-                # so the user doesn't keep getting "Expired session ID"
-                # error pages.  However, it has to be done in the
-                # response object for the error document, which doesn't
-                # exist yet.  Thus, the code that formats SessionError
-                # exceptions -- SessionError.format() by default -- is
-                # responsible for revoking the session cookie.  Yuck.
-                raise SessionError(session_id=id)
-            if (config.check_session_addr and
-                session.get_remote_address() !=
-                request.get_environ("REMOTE_ADDR")):
-                raise SessionError("Remote IP address does not match the "
-                                   "IP address that created the session",
-                                   session_id=id)
-
-        if id is None or session is None:
-            # Generate a session ID and create the session.
-            session = self._create_session(request)
-
+        id = self._get_session_id(config)
+        session = self.get(id) or self._create_session()
         session._set_access_time(self.ACCESS_TIME_RESOLUTION)
         return session
 
-    # get_session ()
+    def maintain_session(self, session):
+        """(session : Session)
 
-    def maintain_session(self, request, session):
-        """maintain_session(request : HTTPRequest, session : Session)
-
-        Maintain session information.  This method is called by
-        SessionPublisher after servicing an HTTP request, just before
-        the response is returned.  If a session contains information it
-        is saved and a cookie dropped on the client.  If not, the
-        session is discarded and the client will be instructed to delete
-        the session cookie (if any).
+        Maintain session information.  This method is called after servicing
+        an HTTP request, just before the response is returned.  If a session
+        contains information it is saved and a cookie dropped on the client.
+        If not, the session is discarded and the client will be instructed
+        to delete the session cookie (if any).
         """
         if not session.has_info():
             # Session has no useful info -- forget it.  If it previously
@@ -296,7 +283,7 @@ class SessionManager:
             # explicitly forget it.
             if session.id and self.has_session(session.id):
                 del self[session.id]
-                self.revoke_session_cookie(request)
+                self.revoke_session_cookie()
             return
 
         if session.id is None:
@@ -304,7 +291,7 @@ class SessionManager:
             # info -- store it and set the session cookie.
             session.id = self._make_session_id()
             self[session.id] = session
-            self.set_session_cookie(request, session.id)
+            self.set_session_cookie(session.id)
 
         elif session.is_dirty():
             # We have already stored this session, but it's dirty
@@ -314,48 +301,47 @@ class SessionManager:
             # repeatedly storing the same object in the same mapping.
             self[session.id] = session
 
-    def _set_cookie(self, request, value, **attrs):
+    def _set_cookie(self, value, **attrs):
         config = get_publisher().config
         name = config.session_cookie_name
         if config.session_cookie_path:
             path = config.session_cookie_path
         else:
-            path = request.environ['SCRIPT_NAME']
+            path = get_request().get_environ('SCRIPT_NAME')
             if not path.endswith("/"):
                 path += "/"
         domain = config.session_cookie_domain
-        request.response.set_cookie(name, value, domain=domain,
-                                    path=path, **attrs)
+        get_response().set_cookie(name, value, domain=domain,
+                                  path=path, **attrs)
         return name
 
-    def set_session_cookie(self, request, session_id):
-        """set_session_cookie(request : HTTPRequest, session_id : string)
+    def set_session_cookie(self, session_id):
+        """(session_id : string)
 
         Ensure that a session cookie with value 'session_id' will be
-        returned to the client via 'request.response'.
+        returned to the client via the response object.
         """
-        self._set_cookie(request, session_id)
-
-    def revoke_session_cookie(self, request):
-        """revoke_session_cookie(request : HTTPRequest)
+        self._set_cookie(session_id)
 
+    def revoke_session_cookie(self):
+        """
         Remove the session cookie from the remote user's session by
-        resetting the value and maximum age in 'request.response'.  Also
-        remove the cookie from 'request' so that further processing of
+        resetting the value and maximum age in the response object.  Also
+        remove the cookie from the request so that further processing of
         this request does not see the cookie's revoked value.
         """
-        cookie_name = self._set_cookie(request, "", max_age=0)
-        if request.cookies.has_key(cookie_name):
-            del request.cookies[cookie_name]
-
-    def expire_session(self, request):
-        """expire_session(request : HTTPRequest)
+        cookie_name = self._set_cookie("", max_age=0)
+        if get_cookie(cookie_name) is not None:
+            del get_request().cookies[cookie_name]
 
+    def expire_session(self):
+        """
         Expire the current session, ie. revoke the session cookie from
         the client and remove the session object from the session
-        manager and from 'request'.
+        manager and from the current request.
         """
-        self.revoke_session_cookie(request)
+        self.revoke_session_cookie()
+        request = get_request()
         try:
             del self[request.session.id]
         except KeyError:
@@ -365,27 +351,50 @@ class SessionManager:
             pass
         request.session = None
 
-    def has_session_cookie(self, request, must_exist=0):
-        """has_session_cookie(request : HTTPRequest,
-                              must_exist : boolean = false)
-           -> boolean
+    def has_session_cookie(self, must_exist=False):
+        """(must_exist : boolean = false) -> bool
 
-        Return true if 'request' already has a cookie identifying a
+        Return true if the request already has a cookie identifying a
         session object.  If 'must_exist' is true, the cookie must
         correspond to a currently existing session; otherwise (the
         default), we just check for the existence of the session cookie
         and don't inspect its content at all.
         """
         config = get_publisher().config
-        id = request.cookies.get(config.session_cookie_name)
+        id = get_cookie(config.session_cookie_name)
         if id is None:
-            return 0
+            return False
         if must_exist:
             return self.has_session(id)
         else:
-            return 1
+            return True
 
-# SessionManager
+    # -- Hooks into the Quixote main loop ------------------------------
+
+    def start_request(self):
+        """
+        Called near the beginning of each request: after the HTTPRequest
+        object has been built, but before we traverse the URL or call the
+        callable object found by URL traversal.
+        """
+        session = self.get_session()
+        get_request().session = session
+        session.start_request()
+
+    def finish_successful_request(self):
+        """Called near the end of each successful request.  Not called if
+        there were any errors processing the request.
+        """
+        session = get_session()
+        if session is not None:
+            self.maintain_session(session)
+        self.commit_changes(session)
+
+    def finish_failed_request(self):
+        """Called near the end of a failed request (i.e. a exception that was
+        not a PublisherError was raised.
+        """
+        self.abort_changes(get_session())
 
 
 class Session:
@@ -404,9 +413,7 @@ class Session:
         or some more complex data structure or object.
       _remote_address : string
         IP address of user owning this session (only set when the
-        session is created -- requests for this session from a different
-        IP address will either raise SessionError or be treated
-        normally, depending on the CHECK_SESSION_ADDR config variable)
+        session is created)
       _creation_time : float
       _access_time : float
         two ways of keeping track of the "age" of the session.
@@ -423,10 +430,10 @@ class Session:
 
     MAX_FORM_TOKENS = 16 # maximum number of outstanding form tokens
 
-    def __init__(self, request, id):
+    def __init__(self, id):
         self.id = id
         self.user = None
-        self._remote_address = request.get_environ("REMOTE_ADDR")
+        self._remote_address = get_request().get_environ("REMOTE_ADDR")
         self._creation_time = self._access_time = time()
         self._form_tokens = [] # queue
 
@@ -440,7 +447,7 @@ class Session:
             return "session %s (no user)" % self.id
 
     def has_info(self):
-        """has_info() -> boolean
+        """() -> boolean
 
         Return true if this session contains any information that must
         be saved.
@@ -448,7 +455,7 @@ class Session:
         return self.user or self._form_tokens
 
     def is_dirty(self):
-        """is_dirty() -> boolean
+        """() -> boolean
 
         Return true if this session has changed since it was last saved
         such that it needs to be saved again.
@@ -460,9 +467,9 @@ class Session:
         a hash file, is_dirty() should probably be an alias or wrapper
         for has_info().  See doc/session-mgmt.txt.
         """
-        return 0
+        return False
 
-    def dump(self, file=None, header=1, deep=1):
+    def dump(self, file=None, header=True, deep=True):
         time_fmt = "%Y-%m-%d %H:%M:%S"
         ctime = strftime(time_fmt, localtime(self._creation_time))
         atime = strftime(time_fmt, localtime(self._access_time))
@@ -474,37 +481,23 @@ class Session:
         file.write('  created %s, last accessed %s' % (ctime, atime))
         file.write('  _form_tokens: %s\n' % self._form_tokens)
 
-    # dump()
-
-
-    # -- Hooks into the Quixote main loop ------------------------------
-
-    def start_request(self, request):
-        """start_request(request : HTTPRequest)
-
+    def start_request(self):
+        """
         Called near the beginning of each request: after the HTTPRequest
-        object has been built and this Session object has been fetched
-        or built, but before we traverse the URL or call the callable
-        object found by URL traversal.
+        object has been built, but before we traverse the URL or call the
+        callable object found by URL traversal.
         """
-        if self.user:
-            request.environ['REMOTE_USER'] = str(self.user)
-
-    def finish_request(self, request):
-        """finish_request(request : HTTPRequest)
-
-        Called near the end of each request: after a callable object has
-        been found and (successfully) called.  Not called if there were
-        any errors processing the request.
-        """
-        pass
-
+        if self.user is not None:
+            get_request().environ['REMOTE_USER'] = str(self.user)
 
     # -- Simple accessors and modifiers --------------------------------
 
     def set_user(self, user):
         self.user = user
 
+    def get_user(self):
+        return self.user
+
     def get_remote_address(self):
         """Return the IP address (dotted-quad string) that made the
         initial request in this session.
@@ -546,7 +539,7 @@ class Session:
     # -- Form token methods --------------------------------------------
 
     def create_form_token(self):
-        """create_form_token() -> string
+        """() -> string
 
         Create a new form token and add it to a queue of outstanding form
         tokens for this session.  A maximum of MAX_FORM_TOKENS are saved.
@@ -560,17 +553,15 @@ class Session:
         return token
 
     def has_form_token(self, token):
-        """has_form_token(token : string) -> boolean
+        """(token : string) -> boolean
 
         Return true if 'token' is in the queue of outstanding tokens.
         """
         return token in self._form_tokens
 
     def remove_form_token(self, token):
-        """remove_form_token(token : string)
+        """(token : string)
 
         Remove 'token' from the queue of outstanding tokens.
         """
         self._form_tokens.remove(token)
-
-# Session
